*-- Dirsubs.prg
*--
*--	Chet Gardiner - 11/5/2002
*--
*-- "Drill Down" and get a table of subdirectories
*--	and the files contained therein
*--
*-- Parameters:
*--		lcDir 		- Top Level Directory
*--		lcTable 	- Table name to return file list in
*--		lcExt		- File Extension List
*--		lcSkeleton  - File Skeleton to search for
*--		lcDirFile 	- if character, save directories in this file
*--		lnLev 		- Number of levels to drill down (0=all)
*--		llDisp		- Don't Display Progress if .t.
*--
*--	Examples:
*--
*--	do dirsubs with "T:\Winpro\", "TWin", "PRG", "*.*", "Tdirs"
*--		will get all .prg files in t:\winpor and save it in twin.dbf and
*--		save the directories in tdirs.dbf
*--
*-- do dirsubs with "V:\PROVFP\", "vPro"
*--		Will get ALL files in v:\provfp\ and save them in vPro.dbf
*--	
*-- do dirsubs with "V:\PROVFP\", "vPro", "FXP|DBF"
*--		Will get ALL .fxp and .dbf files in v:\provfp\ and save them in vPro.dbf
*--
*--	do dirsubs with "\\plsprd41\vol1\ONline\clients\", "cpro", "FXP|DBF|", "ABC*.*", "cdir", 1, .t.
*--		Will get all of the subdirectories for this server's clients in cdir.dbf
*--		(Just first level subdirectories) and won't display progress.  Just looks for files
*--		Starting with "ABC"

parameter lcDir, lcTable, lcExt, lcSkeleton, lcdirfile, lnLev, llDisp

*-- Must have a parameter for the directory to check!!
if type('lcDir') <> "C"
	wait window "Please enter a directory to check!" nowait
	return
endif

*-- Make sure directory entered ends with \
lcdir = alltrim(lcdir)
if right(lcdir,1)<>"\"
	lcdir = lcdir + "\"
endif

*-- Looking for an extension or list of extensions?
if type('lcExt') <> "C"
	lcExt = ""
else
	lcExt = upper(lcExt)
endif

*-- File Skeleton to look for
if type('lcSkeleton') <> "C"
	lcSkeleton = "*.*"
endif

*-- How deep do we go into subdirs?
if type('lnLev') <> "N"
	lnLev = 0
else
	lnLev = lnLev + 1
endif

*-- Write to dirfile.dbf or another named table?
if type('lcTable') <> "C"
	wait window "Output will be put in dirfile.dbf" nowait
	lcTable = "dirfile"
endif

*-- Check for output table
llAddit=.f.
if type('lcTable') = "C"
	if file(lctable+'.dbf')
		*-- Delete the old one?
		lnans = messagebox(lctable+" already exists.  Delete it?", 36)
		if lnans = 6
			*-- Yes!
			if used(lcTable)
				use in (lcTable)
			endif
			erase (lcTable)
		else
			*-- Don't Delete - Add To It!
			llAddit = .t.
			if ! used(lcTable)
				use (lcTable) in 0
			endif
		endif
	endif
endif

*-- Get first level of files into laDir
DIMENSION ladir(1, 1)
lnFilesInDir=l_RunAdir(lcdir)
*-- Got any files/directories?
if lnFilesInDir=0
	wait window "No directories to search.  "+lcDir+" Empty or Doesn't exist!" nowait
	return
endif

*-- Work Cursor for adir results
create cursor ac_work (;
	filename C(60), ;
	len N(15), ;
	date D, ;
	time C(8), ;
	subdir C(240), ;
	ext C(3))

*-- Work Cursor for subdirectory names
create cursor ac_dirs (;
	subdir C(240), ;
	files N(12), ;
	saved N(12), ;
	level N(3), ;
	done L)

*-- Add root/starting directory to subdir table
append blank
replace subdir with lcDir, ;
		level  with 1, ;
		files  with lnFilesInDir

*-- Load Tables
lnLevel = 2
lcD = lcDir

*-- Fill Tables from first ADIR
do filltabl with 1

*-- Now loop through until directories are all done
llDone=.f.
do while ! llDone
	*-- Select next directory not checked
	select ac_dirs
	locate for ! done
	if EOF()
		*-- All done!
		llDone = .t.
	else
		*-- Subdir found yet to do
		*-- Load subdir files
		lnLevel = level + 1
		lcD = alltrim(subdir)
		lnFilesInDir=l_RunAdir(lcD)
		*-- Display message?
		if ! llDisp
			wait window "Check " + lcD + CHR(13) + ;
						alltrim(str(lnFilesInDir))+" Files" nowait
		endif
		*-- Save count and fill table
		replace files with lnFilesInDir
		do filltabl with recno()
		select ac_dirs
	endif
enddo

*-- Add to existing table
if llAddit
	select (lcTable)
	append from dbf('ac_work')
else
	*-- Copy to new table
	select ac_work
	copy to (lcTable)
	use (lcTable)
	index on subdir tag subdir
endif
if used("ac_work")
	use in ac_work
endif

*-- If copied to directory file too
*--	Set relation from directory table
*--	to list of files' table
*-- The directory table may be used by
*--		relating in the other direction
*--		(Files->Dirs) to delete all contents
*--		of subdirs with one command, etc.
if type('lcDirFile') = "C"
	select ac_dirs
	if used(lcDirFile)
		use in (lcDirFile)
	endif
	copy to (lcDirFile)
	use (lcDirFile)
	index on subdir tag subdir
	set relation to subdir into (lcTable)
else
	use in ac_dirs
endif

wait window "Subdirectory Search Finished" nowait

return

*-- *****************************************
*-- *****************************************
*-- Go through ADIR results' Array
*-- Assumes laDir filled by ADIR
*--			lnFilesInDir  = number of laDir elements
*--			lcD = Current Subdirectory
*--			lnLevel = Current Subdir Level?
procedure fillTabl
Parameter lnRec

LOCAL lnx, lcFile

lnCnt = 0
*-- Below lowest level or in Root Dir?
if lnLevel <= lnLev		or ;
   lnLev = 0
  for lnx = 1 to lnFilesInDir
	*-- Is it a directory?
	if "D" $ laDir(lnx, 5)	and ;
	   ! alltrim(laDir(lnx, 1)) $ ".|.."
		*-- Save a Directory
		select ac_dirs
		append blank
		replace subdir with lcd + alltrim(laDir(lnx, 1)) + "\", ;
				level  with lnLevel

	else	
		*-- Is it a file?
		*-- Save a filename
		select ac_work
		lcFile = alltrim(laDir(lnx, 1))
		lcExtn = iif(rat('.',lcFile)<>0, ;
					upper(alltrim(right(lcfile, len(lcfile)-rat('.',lcfile)))), ;
					"")
		if EMPTY(lcExt)			or ;
		   lcExtn $ lcExt
		    lnCnt = lnCnt + 1
			append blank
			replace filename with lcFile, ;
					subdir	 with lcD, ;
					len		 with ladir(lnx, 2), ;
					date	 with ladir(lnx, 3), ;
					time	 with ladir(lnx, 4), ;
					ext		 with lcExtn
		endif
	endif
  endfor
endif

*-- Set this subdir = Done
select ac_dirs
goto lnRec
replace done  with .t., ;
		saved with lnCnt

return

*-- **************************
*-- Load directory information
PROCEDURE l_runAdir
PARAMETERS lcDirect

*-- Directories only
lnDirs  = ADIR(laddir, lcDirect+ "*.*", "D")
*-- All Files
lnFiles = ADIR(ladir, lcDirect+ lcSkeleton, "D")
*-- Add directories to all files array
FOR lnx = 1 TO lnDirs
	if "D" $ ladDir(lnx, 5)	and ;
	   ! alltrim(ladDir(lnx, 1)) $ ".|.."
		*-- Save a Directory
		lnFiles = lnFiles + 1
		DIMENSION laDir(lnFiles, 5)
		FOR lny = 1 TO 5
			laDir(lnFiles, lny) = laddir(lnx, lny)
		ENDFOR
	ENDIF
ENDFOR

RETURN (lnFiles)
